#pragma once

#include <crtdbg.h>
#ifndef ATLASSERT
#define ATLASSERT	_ASSERTE
#endif

#ifdef new
#define _DBG_NEW_DEFINED_
#pragma push_macro("new")
#undef new
#endif


//ATL3CSimpleArrayRsy
template <class T>
class CArray
{
public:
	T* m_aT;
	int m_nSize;
	int m_nAllocSize;

// Construction/destruction
	CArray() : m_aT(NULL), m_nSize(0), m_nAllocSize(0)
	{ }

	~CArray()
	{
		RemoveAll();
	}

// Operations
	int GetSize() const
	{
		return m_nSize;
	}
	BOOL Add(T& t)
	{
		if(m_nSize == m_nAllocSize)
		{
			T* aT;
			int nNewAllocSize = (m_nAllocSize == 0) ? 1 : (m_nSize * 2);
			aT = (T*)realloc(m_aT, nNewAllocSize * sizeof(T));
			if(aT == NULL)
				return FALSE;
			m_nAllocSize = nNewAllocSize;
			m_aT = aT;
		}
		m_nSize++;
		SetAtIndex(m_nSize - 1, t);
		return TRUE;
	}
	BOOL Remove(T& t)
	{
		int nIndex = Find(t);
		if(nIndex == -1)
			return FALSE;
		return RemoveAt(nIndex);
	}
	BOOL RemoveAt(int nIndex)
	{
		ATLASSERT(nIndex >= 0 && nIndex < m_nSize);
		if(nIndex != (m_nSize - 1))
		{
			m_aT[nIndex].~T();
			memmove((void*)&m_aT[nIndex], (void*)&m_aT[nIndex + 1], (m_nSize - (nIndex + 1)) * sizeof(T));
		}
		m_nSize--;
		return TRUE;
	}
	void RemoveAll()
	{
		if(m_aT != NULL)
		{
			for(int i = 0; i < m_nSize; i++)
				m_aT[i].~T();
			free(m_aT);
			m_aT = NULL;
		}
		m_nSize = 0;
		m_nAllocSize = 0;
	}
	T& operator[] (int nIndex) const
	{
		ATLASSERT(nIndex >= 0 && nIndex < m_nSize);
		return m_aT[nIndex];
	}
	T* GetData() const
	{
		return m_aT;
	}
	T* Begin() const
	{
		return m_aT;
	}
	T* End() const
	{
		return m_aT + m_nSize;
	}

// Implementation
	class Wrapper
	{
	public:
		Wrapper(T& _t) : t(_t)
		{
		}
		template <class _Ty>
		void *operator new(size_t, _Ty* p)
		{
			return p;
		}
		T t;
	};
	void SetAtIndex(int nIndex, T& t)
	{
		ATLASSERT(nIndex >= 0 && nIndex < m_nSize);
		new(&m_aT[nIndex]) Wrapper(t);
	}
	int Find(T& t) const
	{
		for(int i = 0; i < m_nSize; i++)
		{
			if(m_aT[i] == t)
				return i;
		}
		return -1;  // not found
	}
};

// for arrays of simple types
template <class T>
class CValArray : public CArray< T >
{
public:
	BOOL Add(T t)
	{
		return CArray< T >::Add(t);
	}
	BOOL Remove(T t)
	{
		return CArray< T >::Remove(t);
	}
	T operator[] (int nIndex) const
	{
		return CArray< T >::operator[](nIndex);
	}
};


//悭񕪒TASY
template <class T, class _Function>
int BinarySearch(const T* pItems, int n, _Function fn)
{
	int st = 0, ed = n;
	while (st < ed - 1) {
		int n = (st + ed) / 2;
		if (fn(pItems[n])) {
			ed = n;
		} else {
			st = n;
		}
	}
	return st;
}

#ifdef _DBG_NEW_DEFINED_
#undef _DBG_NEW_DEFINED_
#pragma pop_macro("new")
#endif
